// Process.cpp
// The main computation goes here

#include <windows.h>
#include <commctrl.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "plugin.h"

// Optimized box blur filter using 2 1-d filter passes and an
// accumulation option to be invariant of the blur radius

static inline void SafeGetPixel(int x, int y, MINIGRAPHIC *src,
				unsigned char *pixel)
{
  if (x<0) x = 0;
  if (y<0) y = 0;
  if (x>=(int)src->width) x = src->width-1;
  if (y>=(int)src->height) y = src->height-1;
  pixel[0] = src->bits[y*src->lineWidth+x*3+0];
  pixel[1] = src->bits[y*src->lineWidth+x*3+1];
  pixel[2] = src->bits[y*src->lineWidth+x*3+2];
}


void ImageIt(int preview, int radius, MINIGRAPHIC *src,
	     MINIGRAPHIC *dst, MINIGRAPHICPARAM *param)
{
	int x, y, i, diam;
	unsigned int sr, sg, sb;
	unsigned char d[3];
	MINIGRAPHIC *tmp;

	if (radius<=0) {
		memcpy(dst->bits, src->bits, src->lineWidth*src->height);
		return;
	}
	
	diam = radius * 2 + 1;

	tmp = param->CreateMiniGraphic( src->width, src->height );

	// Do blur along X
	for (y=0; y<(int)src->height; y++) {
		// Use running sum in sigmas
		sr=sg=sb=0;
		for (i=0; i<diam; i++) {
			SafeGetPixel(0-radius+i, y, src, d);
			sr += (unsigned int)d[0]; sg += (unsigned int)d[1]; sb += (unsigned int)d[2];
		}

		for (x=0; x<(int)src->width; x++) {
			// Update dst pix
			tmp->bits[y*src->lineWidth+x*3+0] = (unsigned char)(sr/diam);
			tmp->bits[y*src->lineWidth+x*3+1] = (unsigned char)(sg/diam);
			tmp->bits[y*src->lineWidth+x*3+2] = (unsigned char)(sb/diam);
	
			// Sub off earliest pixel
			SafeGetPixel(x-radius, y, src, d);
			sr -= (unsigned int)d[0]; sg -= (unsigned int)d[1]; sb -= (unsigned int)d[2];

			// Add in next pixel
			SafeGetPixel(x+radius+1, y, src, d);
			sr += (unsigned int)d[0]; sg += (unsigned int)d[1]; sb += (unsigned int)d[2];
		}
	}

	// Now blur along Y
	for (x=0; x<(int)src->width; x++) {
		// Use running sum in sigmas
		sr=sg=sb=0;
		for (i=0; i<diam; i++) {
			SafeGetPixel(x, 0-radius+i, tmp, d);
			sr += (unsigned int)d[0]; sg += (unsigned int)d[1]; sb += (unsigned int)d[2];
		}

		for (y=0; y<(int)src->height; y++) {
			// Update dst pix
			dst->bits[y*src->lineWidth+x*3+0] = (unsigned char)(sr/diam);
			dst->bits[y*src->lineWidth+x*3+1] = (unsigned char)(sg/diam);
			dst->bits[y*src->lineWidth+x*3+2] = (unsigned char)(sb/diam);
	
			// Sub off earliest pixel
			SafeGetPixel(x, y-radius, tmp, d);
			sr -= (unsigned int)d[0]; sg -= (unsigned int)d[1]; sb -= (unsigned int)d[2];

			// Add in next pixel
			SafeGetPixel(x, y+radius+1, tmp, d);
			sr += (unsigned int)d[0]; sg += (unsigned int)d[1]; sb += (unsigned int)d[2];
		}
	}

	param->FreeMiniGraphic( tmp );
}

